import base64, datetime, urllib.request
import push_bb10.constants as constants
from push_bb10.constants import DeliveryMethod

newLine = '\r\n'


class PushMessage(object):
    def __init__(self, id_, message):
        if id_ is None or id_.strip() == '':
            raise ValueError('id_ is empty')
        self.id_ = id_
        self.message = message
        self.recipients = []
        self.delivery_method = DeliveryMethod.notspecified


    def add_recipient(self, token):
        """ Adds a new recipient to the message """
        if not token in self.recipients:
            self.recipients.append(token)


    def add_all_recipients(self, tokens):
        """ Adds multiple recipients at once """
        for token in tokens:
            self.add_recipient(token)


    def to_PAP_message(self, options):
        """ Converts the push message to the correct Push Access Control Protocol message. """
        if len(self.recipients) == 0:
            raise Exception('Please, add some recipients before sending the push message')

        body = '--' + constants.boundary + newLine
        body += 'Content-Type: application/xml; charset=UTF-8' + newLine
        body += newLine
        body += '<?xml version="1.0"?>' + newLine
        body += '<!DOCTYPE pap PUBLIC "-//WAPFORUM//DTD PAP 2.1//EN" "http://www.openmobilealliance.org/tech/DTD/pap_2.1.dtd">' + newLine
        body += '<pap>' + newLine

        attributes = 'push-id="' + self.id_ + '" ';
        for key in options:
            attributes += key + '="' + options[key] + '" '

        body += '    <push-message ' + attributes.strip() + '>' + newLine
        for token in self.recipients:
            body += '        <address address-value="' + token + '"/>' + newLine
        body += '        <quality-of-service delivery-method="' + self.delivery_method.name + '" />' + newLine
        body += '    </push-message>' + newLine
        body += '</pap>' + newLine
        body += '--' + constants.boundary + newLine
        body += 'Content-Encoding: binary' + newLine
        body += 'Content-Type: text/html' + newLine
        body += 'Push-Message-ID: ' + self.id_ + newLine
        body += newLine
        body += self.message + newLine
        body += newLine
        body += '--' + constants.boundary + '--' + newLine
        body += newLine

        return body





class PushInitiator(object):
    def __init__(self, application_id, password, content_provider_id, evaluation):
        self.app_id = application_id
        auth_token = str(application_id + ':' + password).encode('ascii')
        self.auth_token = base64.b64encode(auth_token).decode('utf-8')
        self.content_provider_id = content_provider_id
        self.is_evaluation = evaluation or False


    def push(self, message):
        """ Sends the message to the Push Protocol Gateway """
        delivery_before = delivery_timestamp()
        body = message.to_PAP_message({
            'source-reference': self.app_id,
            'deliver-before-timestamp': delivery_before.replace(microsecond=0).isoformat('T') + 'Z'
        })
        host = None
        if self.is_evaluation:
            host = 'https://' + self.content_provider_id + '.' + constants.eval_url
        else:
            host = 'https://' + self.content_provider_id + '.' + constants.production_url

        headers = {
            'Content-Type': 'multipart/related; boundary=' + constants.boundary + '; type=application/xml',
            'Content-length': len(body),
            'Authorization': 'Basic ' + self.auth_token
        }
        port = 443
        request = urllib.request.Request(host + ':' + str(port) + '/mss/PD_pushRequest', data=body.encode('ascii'), headers=headers)
        response = urllib.request.urlopen(request)
        return response.read().decode('utf8')


def delivery_timestamp():
    """ Returns the current time plus 8 hours ahead """
    return datetime.datetime.utcnow() + datetime.timedelta(hours=8)
